from database import Database, ImageProcessedState, ImageUserAction, BoundingBox, StickUserAction
import logging
from unittest import TestCase
from datetime import datetime


class TestDatabaseInsert(TestCase):

    def setUp(self):
        self.db = Database(':memory:')

    def tearDown(self):
        self.db.close()

    def test_tables_created(self):
        res = self.db.cur.execute('SELECT name FROM sqlite_schema').fetchall()
        self.assertEqual(len(res), 5)
        self.assertIn(('Experiments',), res)
        self.assertIn(('Cameras',), res)
        self.assertIn(('Sticks',), res)
        self.assertIn(('Images',), res)
        self.assertIn(('Stick_Images',), res)

    def test_insert_experiment(self):
        # The table is empty.
        res = self.db.cur.execute('SELECT * FROM Experiments').fetchone()
        self.assertIsNone(res)

        self.db.insert_experiment(
            'location', 1234, datetime(1, 1, 1), datetime(1, 1, 1), 0.0, 0.0)

        # The row is inserted correctly.
        res = self.db.cur.execute('SELECT * FROM Experiments').fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 1)
        self.assertEqual(res[0][1], 'location')
        self.assertEqual(res[0][2], 1234)
        self.assertEqual(res[0][3], str(datetime(1, 1, 1)))
        self.assertEqual(res[0][4], str(datetime(1, 1, 1)))
        self.assertEqual(res[0][5], 0.0)
        self.assertEqual(res[0][6], 0.0)

    def test_insert_camera(self):
        # The table is empty.
        res = self.db.cur.execute('SELECT * FROM Cameras').fetchone()
        self.assertIsNone(res)

        experiment_id = self.db.insert_experiment(
            'location', 1234, datetime(1, 1, 1), datetime(1, 1, 1), 0.0, 0.0)

        # The method returns None when the parent table doesn't contain valid id.
        res = self.db.insert_camera(2, 'cam')
        self.assertIsNone(res)

        # The row is inserted correctly.
        self.db.insert_camera(experiment_id, 'cam')
        res = self.db.cur.execute('SELECT * FROM Cameras').fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 1)
        self.assertEqual(res[0][1], 1)
        self.assertEqual(res[0][2], 'cam')

        # The row is deleted after parent experiment is deleted
        self.db.cur.execute(
            'DELETE FROM Experiments WHERE id=?', (experiment_id,))
        self.db.con.commit()
        res = self.db.cur.execute('SELECT * FROM Cameras').fetchone()
        self.assertIsNone(res)

    def test_insert_image(self):
        # The table is empty.
        res = self.db.cur.execute('SELECT * FROM Images').fetchone()
        self.assertIsNone(res)

        experiment_id = self.db.insert_experiment(
            'location', 1234, datetime(1, 1, 1), datetime(1, 1, 1), 0.0, 0.0)
        camera_id = self.db.insert_camera(experiment_id, 'cam')

        # The method returns None when the parent table doesn't contain valid id.
        res = self.db.insert_image(2, 'path/to/image', ImageProcessedState.PROCESSED,
                                   datetime(1, 1, 1), True, True, None, ImageUserAction.NO_ACTION)
        self.assertIsNone(res)

        # The row is inserted correctly.
        self.db.insert_image(camera_id, 'path/to/image', ImageProcessedState.PROCESSED,
                             datetime(1, 1, 1), True, True, None, ImageUserAction.NO_ACTION)
        res = self.db.cur.execute('SELECT * FROM Images').fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 1)
        self.assertEqual(res[0][1], camera_id)
        self.assertEqual(res[0][2], 'path/to/image')
        self.assertEqual(res[0][3], ImageProcessedState.PROCESSED)
        self.assertEqual(res[0][4], str(datetime(1, 1, 1)))
        self.assertTrue(res[0][5])
        self.assertTrue(res[0][6])
        self.assertIsNone(res[0][7])
        self.assertEqual(res[0][8], ImageUserAction.NO_ACTION)

        # The row is deleted after parent experiment is deleted
        self.db.cur.execute(
            'DELETE FROM Experiments WHERE id=?', (experiment_id,))
        res = self.db.cur.execute('SELECT * FROM Images').fetchone()
        self.assertIsNone(res)

    def test_insert_stick(self):
        # The table is empty.
        res = self.db.cur.execute('SELECT * FROM Sticks').fetchone()
        self.assertIsNone(res)

        experiment_id = self.db.insert_experiment(
            'location', 1234, datetime(1, 1, 1), datetime(1, 1, 1), 0.0, 0.0)
        camera_id = self.db.insert_camera(experiment_id, 'cam')
        image_id = self.db.insert_image(camera_id, 'path/to/image', ImageProcessedState.PROCESSED,
                                        datetime(1, 1, 1), True, True, None, ImageUserAction.NO_ACTION)

        # The method returns None when the parent table doesn't contain valid id.
        res = self.db.insert_stick(2, 2, 'stick_1', 42, 2,
                                   BoundingBox(0, 0, 10, 10), 2)
        self.assertIsNone(res)

        # The row is inserted correctly.
        self.db.insert_stick(experiment_id, camera_id, 'stick_1', 42,
                             camera_id, BoundingBox(0, 0, 10, 10), image_id)
        res = self.db.cur.execute('SELECT * FROM Sticks').fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 1)
        self.assertEqual(res[0][1], experiment_id)
        self.assertEqual(res[0][2], camera_id)
        self.assertEqual(res[0][3], 'stick_1')
        self.assertEqual(res[0][4], 42)
        self.assertEqual(res[0][5], camera_id)
        self.assertEqual(res[0][6], '(0, 0, 10, 10)')
        self.assertEqual(res[0][7], image_id)

        # The row is deleted after parent experiment is deleted
        self.db.cur.execute(
            'DELETE FROM Experiments WHERE id=?', (experiment_id,))
        res = self.db.cur.execute('SELECT * FROM Sticks').fetchone()
        self.assertIsNone(res)

    def test_insert_stick_image(self):
        # The table is empty.
        res = self.db.cur.execute('SELECT * FROM Stick_Images').fetchone()
        self.assertIsNone(res)

        experiment_id = self.db.insert_experiment(
            'location', 1234, datetime(1, 1, 1), datetime(1, 1, 1), 0.0, 0.0)
        camera_id = self.db.insert_camera(experiment_id, 'cam')
        image_id = self.db.insert_image(camera_id, 'path/to/image', ImageProcessedState.PROCESSED,
                                        datetime(1, 1, 1), True, True, None, ImageUserAction.NO_ACTION)
        stick_id = self.db.insert_stick(experiment_id, camera_id, 'stick_1', 42,
                                        camera_id, BoundingBox(0, 0, 10, 10), image_id)

        # The method returns None when the parent table doesn't contain valid id.
        res = self.db.insert_stick_image(
            2, 2, 2, BoundingBox(0, 0, 10, 10), 0.5, True, StickUserAction.NO_ACTION)
        self.assertIsNone(res)

        # The row is inserted correctly.
        self.db.insert_stick_image(
            stick_id, camera_id, image_id, BoundingBox(0, 0, 10, 10), 0.5, True, StickUserAction.NO_ACTION)
        res = self.db.cur.execute('SELECT * FROM Stick_Images').fetchall()
        self.assertEqual(len(res), 1)
        self.assertEqual(res[0][0], 1)
        self.assertEqual(res[0][1], stick_id)
        self.assertEqual(res[0][2], camera_id)
        self.assertEqual(res[0][3], image_id)
        self.assertEqual(res[0][4], '(0, 0, 10, 10)')
        self.assertEqual(res[0][5], 0.5)
        self.assertEqual(res[0][6], True)
        self.assertEqual(res[0][7], StickUserAction.NO_ACTION)

        # The row is deleted after parent experiment is deleted
        self.db.cur.execute(
            'DELETE FROM Experiments WHERE id=?', (experiment_id,))
        res = self.db.cur.execute('SELECT * FROM Stick_Images').fetchone()
        self.assertIsNone(res)
